/* GScript
 * gscript.h:
 *
 * Copyright (C) 1999 Red Hat Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* The API here is based fairly closely on Microsoft's Uniscript
 * API. Differences here:
 * 
 * - Memory management is more convenient (GScript handles 
 *   all buffer reallocation)
 * - Unicode strings are represented in UTF-8
 * - Representation of fonts and glyphs is abstracted to be
 *   rendering-system dependent.
 */

#ifndef __GSCRIPT_H__
#define __GSCRIPT_H__

#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GScriptAnalysis GScriptAnalysis;
typedef struct _GScriptItem GScriptItem;
typedef struct _GScriptContext GScriptContext;
typedef struct _GScriptLangRange GScriptLangRange;
typedef struct _GScriptLogAttr GScriptLogAttr;

typedef struct _GScriptFont GScriptFont;
typedef struct _GScriptFontClass GScriptFontClass;
typedef struct _GScriptCFont GScriptCFont;
typedef struct _GScriptCFontClass GScriptCFontClass;

typedef struct _GGlyph GGlyph;
typedef struct _GGlyphGeometry GGlyphGeometry;
typedef struct _GGlyphVisAttr GGlyphVisAttr;
typedef struct _GGlyphString GGlyphString;

typedef struct _GScriptEngineInfo GScriptEngineInfo;
typedef struct _GScriptEngineRange GScriptEngineRange;
typedef struct _GScriptEngine GScriptEngine;
typedef struct _GScriptEngineLang GScriptEngineLang;
typedef struct _GScriptEngineShape GScriptEngineShape;

/* 64'ths of a point - 1/4608 in, 5.51 * 10^-5 in. */
typedef guint32 GGlyphUnit;

/* Information about a segment of text with a consistent
 * shaping/language engine and bidirectional level
 */
struct _GScriptAnalysis {
  GScriptEngineShape *shape_engine;
  GScriptEngineLang *lang_engine;
  guint8 level;
};

struct _GScriptItem {
  gint offset;
  gint length;
  gint num_chars;
  GScriptAnalysis analysis;
};

/* Sort of like a GC - application set information about how
 * to handle scripts
 */

typedef enum {
  G_SCRIPT_DIRECTION_LTR,
  G_SCRIPT_DIRECTION_RTL,
  G_SCRIPT_DIRECTION_TTB
} GScriptDirection;

struct _GScriptContext {
  gchar *lang;
  gchar *render_type;
  GScriptDirection direction;
};

/* Language tagging information
 */
struct _GScriptLangRange
{
  gint start;
  gint length;
  gchar *lang;
};

/* Break a string of Unicode characters into segments with
 * consistent shaping/language engine and bidrectional level.
 * Returns a GList of GScriptItem's
 */
GList *g_script_itemize (GScriptContext   *context, 
                         gchar            *text, 
                         gint              length,
			 GScriptLangRange *lang_info,
			 gint              n_langs);

/* Logical attributes of a character
 */
struct _GScriptLogAttr {
  guint is_break : 1;  /* Break in front of character */
  guint is_white : 1;
  guint is_char_stop : 1;
  guint is_word_stop : 1;
  /* Uniscript has is_invalid  */
};

/* Determine information about cluster/word/line breaks in a string
 * of Unicode text.
 */
void g_script_break (gchar           *text, 
                     gint             length, 
                     GScriptAnalysis *analysis, 
                     GScriptLogAttr  *attrs);

/*
 * FONT OPERATIONS
 */

/* This structure represents a logical font */
struct _GScriptFont {
  GScriptFontClass *klass;

  /*< private >*/
  gint ref_count;
  GData *data;
};

struct _GScriptFontClass {
  void (*destroy) (GScriptFont *font);
};
 
void     g_script_font_init   (GScriptFont *font);
void     g_script_font_ref    (GScriptFont *font);
void     g_script_font_unref  (GScriptFont *font);
gpointer g_script_font_get_data (GScriptFont    *font,
		    	         gchar          *key);
void     g_script_font_set_data (GScriptFont   *font,
				 gchar         *key,
				 gpointer       data,
				 GDestroyNotify destroy_func);

/* A component of a font. The contents of this are completely
 * rendering system dependent
 */
struct _GScriptCFont {
  GScriptCFontClass *klass;

  /*< private >*/
  gint ref_count;
};

struct _GScriptCFontClass {
  void (*destroy) (GScriptCFont *cfont);
};

void g_script_cfont_init  (GScriptCFont *font);
void g_script_cfont_ref   (GScriptCFont *font);
void g_script_cfont_unref (GScriptCFont *font);


/*
 * GLYPH STORAGE
 */

/* A index of a glyph into a GScriptCFont. Rendering system dependent
 */
typedef guint32 GGlyphIndex;

/* A single glyph 
 */
struct _GGlyph
{
  GGlyphIndex  glyph;
  GScriptCFont *font;
};

/* Positioning information about a glyph
 */
struct _GGlyphGeometry
{
  GGlyphUnit width;
  GGlyphUnit x_offset;  
  GGlyphUnit y_offset;
};

/* Visual attributes of a glyph
 */
struct _GGlyphVisAttr
{
  guint is_cluster_start : 1;
};

/* A string of glyphs with positional information and visual attributes -
 * ready for drawing
 */
struct _GGlyphString {
  gint num_glyphs;
  GGlyph *glyphs;
  GGlyphGeometry *geometry;
  GGlyphVisAttr *attrs;

  /* This is a memory inefficient way of representing the
   * information here - each value gives the character index
   * of the start of the cluster to which the glyph belongs.
   */
  gint *log_clusters;

  /*< private >*/
  gint space;
};

GGlyphString *g_glyph_string_new  (void);
void          g_glyph_string_set_size (GGlyphString *string, gint new_len);
void          g_glyph_string_free (GGlyphString *string);


/* Turn a string of characters into a string of glyphs
 */
void g_script_shape (GScriptFont      *font, 
                     gchar            *text, 
                     gint              length, 
                     GScriptAnalysis *analysis,
                     GGlyphString     *glyphs);

/* [ g_script_place - subsume into g_script_shape? ] */

GList *g_script_reorder_items (GList *logical_items);

/* Take a GGlyphString and add justification to fill to a
 * given width
 */
void g_script_justify (GGlyphString   *glyphs,
                       gint            new_line_width,
                       gint            min_kashida_width);

/* For selection/cursor positioning - turn a character position into a
 * X position.
 */
void g_script_cp_to_x (gchar           *text,
                       gint             length,
		       GScriptAnalysis *analysis,
                       GGlyphString    *glyphs,
                       gint             char_pos,
		       gboolean         trailing,
            	       gint            *x_pos);

/* For selection/cursor positioning - turn a X position into a
 * character position
 */
void g_script_x_to_cp (gchar           *text,
                       gint             length,
                       GScriptAnalysis *analysis,
                       GGlyphString    *glyphs,
		       gint             x_pos,
                       gint            *char_pos, 
                       gint            *trailing);

/* Module API */

#define G_SCRIPT_ENGINE_TYPE_LANG "GScriptEngineLang"
#define G_SCRIPT_ENGINE_TYPE_SHAPE "GScriptEngineShape"

#define G_SCRIPT_RENDER_TYPE_NONE "GScriptRenderNone"

struct _GScriptEngineRange 
{
  guint32 start;
  guint32 end;
  gchar *langs;
};

struct _GScriptEngineInfo
{
  gchar *id;
  gchar *engine_type;
  gchar *render_type;
  GScriptEngineRange *ranges;
  gint n_ranges;
};

struct _GScriptEngine
{
  gchar *id;
  gchar *type;
  gint length;
};

struct _GScriptEngineLang
{
  GScriptEngine engine;
  void (*script_break) (gchar            *text,
			gint             len,
			GScriptAnalysis *analysis,
			GScriptLogAttr  *attrs);
};

struct _GScriptEngineShape {
  GScriptEngine engine;
  void (*script_shape) (GScriptFont     *font,
			gchar           *text,
			gint             length,
			GScriptAnalysis *analysis,
			GGlyphString    *glyphs);
};

/* A module should export the following functions */

void script_engine_list (GScriptEngineInfo **engines, gint *n_engines);
GScriptEngine *script_engine_load (const gchar *id);
void script_engine_unload (GScriptEngine *engine);
			   
#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GSCRIPT_H__ */
